<?php
/**
 *    
 *
 * @package DB
 * @version 0.0.1
 * @author   , aka vasa_c
 * @license LGPL
 * @uses    mysqli
 * @PHP5Only
 */

class DB extends mysqli
{

    const baseName = 'base';

    /**
     * .  mysqli  ,      
     *
     * @exception DBExceptionConnect
     *
     * @param string $host      [optional]
     * @param string $username  [optional]
     * @param string $passwd    [optional]
     * @param string $dbname    [optional]
     * @param int    $port      [optional]
     * @param string $socket    [optional]
     */
    public function __construct($host = null, $username = null, $passwd = null, $dbname = null, $port = null, $socket = null)
    {
        @parent::__construct($host, $username, $passwd, $dbname, $port, $socket);
        if (mysqli_connect_errno()) {
            throw new DBExceptionConnect(mysqli_connect_error(), mysqli_connect_errno());
        }
    }

    /**
     *    
     *
     * @param  string $pattern sql-  -  
     * @param  array  $data      
     * @param  string $fetch    
     * @param  string $prefix  [optional]   
     * @return mixed      
     * @exception DBExceptionQuery   
     */
    public function query($pattern, $data = null, $fetch = null, $prefix = null)
    {
        $prefix = $prefix ? $prefix : $this->tablePrefix;
        $query = $data ? self::makeQuery($pattern, $data, $prefix) : $pattern;
        $result = parent::query($query, MYSQLI_STORE_RESULT);
        if ($this->queryDebug) {
            print '<pre'.(!$result?' style="color:red"':'').'>'.htmlSpecialChars($query).'</pre>';
        }
        if ($this->errno) {
            throw new DBExceptionQuery($query, $this->errno, $this->error);
        }
        $return = $this->fetch($result, $fetch);
        if ((!is_object($return)) && (is_object($result))) {
            $result->free();
        }
        return $return;
    }

    /**
     *  
     *
     * @param string $pattern -  
     * @param array  $data      
     * @param string $prefix  [optional]  
     */
    public function makeQuery($pattern, $data, $prefix = '')
    {
        $this->_mqPH = $data;
        $this->_mqPrefix = $prefix;
        $q = @preg_replace_callback('/\?([int?ca]?[ina]?);?/', Array($this, '_makeQuery'), $pattern);
        if (sizeOf($this->_mqPH) > 0) {
            throw new DBExceptionDataMuch();
        }
        return $q;
    }

    /**
     *     
     *
     * @param  mysqli_result $result 
     * @param  string        $fetch  
     * @return mixed
     */
    public function fetch($result, $fetch)
    {
        $fetch = strToLower($fetch);
        if ((!$fetch) || ($fetch == 'no')) {
            return $result;
        }
        if ($fetch == 'id') {
            return $this->insert_id;
        }
        if ($fetch == 'ar') {
            return $this->affected_rows;
        }
        $numRows = $result->num_rows;
        if ($fetch == 'num') {
            return $numRows;
        }
        if (($fetch == 'kcol' || $fetch == 'kassoc') && $result->field_count < 2) $fetch = 'col';
        if ($fetch == 'row') {
            $A = Array();
            for ($i = 0; $i < $numRows; $i++) {
                $A[] = $result->fetch_row();
            }
            return $A;
        }
        if ($fetch == 'assoc') {
            $A = Array();
            for ($i = 0; $i < $numRows; $i++) {
                $A[] = $result->fetch_assoc();
            }
            return $A;
        }
        if ($fetch == 'col') {
            $A = Array();
            for ($i = 0; $i < $numRows; $i++) {
                $r = $result->fetch_row();
                $A[] = $r[0];
            }
            return $A;
        }
        if ($fetch == 'kassoc') {
            $A = Array();
            for ($i = 0; $i < $numRows; $i++) {
            	$r = $result->fetch_assoc();
                $A[current($r)] = $r;
            }
            return $A;
        }
        if ($fetch == 'kcol') {
            $A = Array();
            for ($i = 0; $i < $numRows; $i++) {
                $r = $result->fetch_row();
                $A[$r[0]] = $r[1];
            }
            return $A;
        }
        if ($fetch == 'irow') {
            return new DBResultRow($result);
        }
        if ($fetch == 'iassoc') {
            return new DBResultAssoc($result);
        }
        if ($fetch == 'icol') {
            return new DBResultCol($result);
        }
        if ($numRows == 0) {
            return false;
        }
        if ($fetch == 'rowrow') {
            return $result->fetch_row();
        }
        if ($fetch == 'rowassoc') {
            return $result->fetch_assoc();
        }
        if ($fetch == 'el') {
            $r = $result->fetch_row();
            return $r[0];
        }
        return true;
    }

    /**
     *   
     *
     * @param string $prefix
     */
    public function setPrefix($prefix)
    {
        $this->tablePrefix = $prefix;
        return true;
    }

    /**
     *   
     *
     * @param bool $debug
     */
    public function setDebug($debug = true)
    {
        $this->queryDebug = $debug;
        return true;
    }

    /**
     *       
     *
     * @param  string $host
     * @param  string $username
     * @param  string $passwd
     * @param  string $dbname
     * @param  string $name [optional] 
     * @return DB
     */
    public static function makeDB($host, $username, $passwd, $dbname, $name = self::baseName)
    {
        if (isSet(self::$dbList[$name])) {
            throw new DBExceptionDBAlready($name);
        }
        return (self::$dbList[$name] = new self($host, $username, $passwd, $dbname));
    }

    /**
     *     
     *
     * @param DB $db
     * @param string $name
     */
    public static function setDB(DB $db, $name = self::baseName)
    {
        if (isSet(self::$dbList[$name])) {
            throw new DBExceptionDBAlready($name);
        }
        self::$dbList[$name] = $db;
        return true;
    }

    /**
     *     
     *
     * @param string $name
     */
    public static function getDB($name = self::baseName)
    {
        if (!isSet(self::$dbList[$name])) {
            throw new DBExceptionDBNotFound($name);
        }
        return self::$dbList[$name];
    }

    /**
     *      
     *
     * @param  string $pattern
     * @param  array  $data
     * @param  string $fetch
     * @param  string $prefix
     * @param  string $name
     * @return mixed
     */
    public static function queryDB($pattern, $data = null, $fetch = null, $prefix = '', $name = self::baseName)
    {
        return self::getDB($name)->query($pattern, $data, $fetch, $prefix);
    }

/*** PRIVATE: ***/

    /**
     *     
     *
     * @param  array  $ph
     * @return string
     */
    private function _makeQuery($ph)
    {
        if ($ph[1] == '?') {
            return '?';
        }
        if (sizeOf($this->_mqPH) == 0) {
            throw new DBExceptionDataNotEnough();
        }
        $el = array_shift($this->_mqPH);
        switch ($ph[1]) {
            case ('i'): return intVal($el);
            case ('t'): return '`'.$this->_mqPrefix.$el.'`';
            case ('c'):
                if (is_array($el)) {
                    return '`'.$this->_mqPrefix.$el[0].'`.`'.$el[1].'`';
                }
                return '`'.$el.'`';
            case ('n'): return is_null($el) ? 'NULL' : ('"'.$this->real_escape_string($el).'"');
            case ('ni'):
            case ('in'): return is_null($el) ? 'NULL' : intVal($el);
            case ('a'):
                foreach ($el as &$e) {
                    $e = '"'.$this->real_escape_string($e).'"';
                }
                return implode(',', $el);
            case ('ai'):
            case ('ia'):
                foreach ($el as &$e) {
                    $e = intVal($e);
                }
                return implode(',', $el);
        }
        return '"'.$this->real_escape_string($el).'"';
    }


/*** VARS: ***/

    /**
     *    
     *
     * @var string
     */
    protected $tablePrefix = '';

    /**
     *  
     *
     * @var bool
     */
    protected $queryDebug = false;

    /**
     *   
     *
     * @var array
     */
    protected static $dbList = Array();

    /*   */
    private $_mqPH;
    private $_mqPrefix;
}


/**
 *      
 */

/**
 *  
 */
class DBException extends Exception {}

/**
 *  
 */
class DBExceptionConnect extends DBException {}

/**
 *   
 */
class DBExceptionQuery extends DBException
{
    public function __construct($query, $errno, $error)
    {
        $this->query = $query;
        $this->errno = $errno;
        $this->error = $error;
    }

    public function query() {return $this->query;}
    public function errno() {return $this->errno;}
    public function error() {return $this->error;}

    public function __toString()
    {
        return 'DB Error. Query="'.htmlSpecialChars($this->query).'" error = '.$this->errno.' "'.htmlSpecialChars($this->error()).'"';
    }

    private $query, $errno, $error;

}

/**
 *      
 */
class DBExceptionData extends DBException
{
    protected $message = 'DB Data Exception';
}

/**
 *   
 */
class DBExceptionDataMuch extends DBExceptionData
{
    protected $message = 'It is too much data';
}

/**
 *   
 */
class DBExceptionDataNotEnough extends DBExceptionData
{
    protected $message = 'It is not enough data';
}

/**
 *         DB
 */
class DBExceptionDB extends DBException {}
class DBExceptionDBAlready  extends DBExceptionDB {}
class DBExceptionDBNotFound extends DBExceptionDB {}


/**
 *    ()
 *
 */
abstract class DBResult implements Iterator, ArrayAccess
{

    public function __construct($result)
    {
        $this->result  = $result;
        $this->numRows = $result->num_rows;
    }

    public function __destruct()
    {
        if (is_resource($this->result)) {
            $this->result->free();
        }
    }

    public function rewind()
    {
        $this->count = 0;
        return true;
    }

    public function current()
    {
        $this->result->data_seek($this->count);
        return $this->getEl();
    }

    public function key()
    {
        return $this->count;
    }

    public function next()
    {
        $this->count++;
        return $this->current();
    }

    public function valid()
    {
        if ($this->count >= $this->numRows) {
            return false;
        }
        return true;
    }

    public function count()
    {
        return $this->numRows;
    }

    public function get($num, $index = false)
    {
        if ($num >= $this->numRows) {
            return false;
        }
        $this->result->data_seek($num);
        $r = $this->getEl();
        if ($index === false) {
            return $r;
        }
        if (!is_array($r)) {
            return false;
        }
        if (!isSet($r[$index])) {
            return false;
        }
        return $r[$index];
    }

    public function offsetGet($offset)
    {
        return $this->get($offset);
    }
    public function offsetSet($offset, $value)
    {
        return false;
    }
    public function offsetExists($offset)
    {
        return (($offset >= 0) && ($offset < $numRows));
    }
    public function offsetUnset($offset)
    {
        return false;
    }


    abstract protected function getEl();

    protected $result, $numRows, $count = 0;
}

class DBResultRow extends DBResult
{
    protected function getEl()
    {
        return $this->result->fetch_row();
    }
}

class DBResultAssoc extends DBResult
{
    protected function getEl()
    {
        return $this->result->fetch_assoc();
    }
}

class DBResultCol extends DBResult
{
    protected function getEl()
    {
        $r = $this->result->fetch_row();
        return $r[0];
    }
}

?>
