<?php
/**
 *        
 */

class DB_SE extends DB
{
	/**
	 *   
	 */
	protected $rollback = false;
	
	/**
	 *   
	 */
	protected $transaction = false;
	
	/**
	 *    
	 */
	public function Transaction()
	{
		if ($this->transaction) {
			$this->endTransaction();
		} else {
			$this->startTransaction();
		}
		return true;
	}
	
	/**
	 *   
	 */
	protected function startTransaction()
	{
		if ($this->autocommit(false)) {
			$this->transaction = true;
		}
		else {
			throw new DBExceptionTransactionFailed($this->errno, $this->error);
		}
	}
	
	/**
	 *     
	 */
	protected function endTransaction()
	{
		if ($this->transaction)
		{
			if ($this->rollback)
			{
				if (!$this->rollback()) {
					throw new DBExceptionRollbackFailed($this->errno, $this->error);
				}
				$this->rollback = false;
			}
			else
			{
				if (!$this->commit()) {
					throw new DBExceptionCommitFailed($this->errno, $this->error);
				}
			}
			$this->transaction = false;
			$this->autocommit(true);
		}
	}
	
	/**
	 *      
	 */
	public function resetTransaction()
	{
		$this->transaction = false;
		$this->autocommit(true);
		$this->rollback = false;
	}
	
	/**
	 *     .
	 * ! NULL-   INT=0,
	 *     CONSTRAINT   
	 *  INSERT  UPDATE   .
	 *
	 * @param unknown_type $value
	 * @return unknown
	 */
	protected function smartEscape($value)
	{
		return (is_numeric($value) ? $value : ($value === null ? 0 : "'".$this->real_escape_string($value)."'") );
	}
	
	/**
	 *   VALUES  WHERE
	 * ! NULL-   INT=0   self::smartEscape().
	 *
	 * @param array		$keys		 (  => )
	 * @param string	$separator	    (, AND  OR)
	 * @param string	$mode		k[eys]|v[alues]|[b[oth]]  , ,     
	 * @return string
	 */
	protected function MakeSQLFromArray($keys, $separator, $mode = '')
	{
		$ret = $sep = '';
		foreach($keys as $key => $value)
		{
			$ret .= $sep;
			switch ($mode)
			{
				case 'k':
				case 'keys':
					$ret .= '`'.$key.'`';
					break;
				case 'v':
				case 'values':
					$ret .= $this->smartEscape($value);
					break;
				case 'b':
				case 'both':
				default:
					$ret .= '`'.$key.'`'."=".$this->smartEscape($value);
			}
			$sep = $separator;
		}
		return $ret;
	}

	/**
	 *    ,    
	 *
	 * @param string	$table	 
	 * @param array		$clause	 (  => )
	 * @return boolean query result
	 */
	public function Delete($table, $clause)
	{
		if (sizeof($clause) == 0) {
			throw new DBExceptionData('No keys for WHERE clause');
		}
		$where = $this->MakeSQLFromArray($clause, ' AND ');
		$sql = 'DELETE FROM ?t WHERE '.$where;
		if ($this->transaction)
		{
			try
			{
				$result = $this->query($sql, array($table));
				$this->rollback = !$result;
			} catch (DBException $e) {
				$this->rollback = true;
			}
			if ($this->rollback) {
				throw new DBExceptionRollbackTrigger();
			}
		}
		return $result;
	}
	
	/**
	 *     
	 *
	 * @param string	$table	 
	 * @param array 	$data	 (  => )
	 * @return boolean query result
	 */
	public function Insert($table, $data)
	{
		if (sizeof($data) == 0) {
			throw new DBExceptionData('No data for INSERT');
		}
		$sql = 'INSERT INTO ?t ('.$this->MakeSQLFromArray($data, ',', 'k')
				.') VALUES('.$this->MakeSQLFromArray($data, ',', 'v').')';
		if ($this->transaction)
		{
			try
			{
				$result = $this->query($sql, array($table));
				$this->rollback = !$result;
			} catch (DBException $e) {
				$this->rollback = true;
			}
			if ($this->rollback) {
				throw new DBExceptionRollbackTrigger();
			}
		}
		return $result;
	}
	
	/**
	 *    
	 *
	 * @param string	$table	 
	 * @param array		$data	 (  => )
	 * @param array		$clause	 (  => )
	 * @return boolean query result
	 */
	public function Update($table, $data, $clause)
	{
		if (sizeof($data) == 0) {
			throw new DBExceptionData('No data for UPDATE');
		}
		if (sizeof($clause) == 0) {
			throw new DBExceptionData('No keys for WHERE clause');
		}
		$sql = 'UPDATE ?t SET '.$this->MakeSQLFromArray($data, ', ')
				.' WHERE '.$this->MakeSQLFromArray($clause, ' AND ');
		if ($this->transaction)
		{
			try
			{
				$result = $this->query($sql, array($table));
				$this->rollback = !$result;
			} catch (DBException $e) {
				$this->rollback = true;
			}
			if ($this->rollback) {
				throw new DBExceptionRollbackTrigger();
			}
		}
		return $result;
	}

	
	/**
	 *             
	 *
	 * @param string	$table	 
	 * @param array		$data	 (  => ), 
	 * 							        (UNIQUE  PRIMARY KEY)
	 * @return boolean query result
	 */
	public function SmartInsert($table, $data)
	{
		if (sizeof($data) == 0) {
			throw new DBExceptionData('No data for INSERT');
		}
		$sql = 'INSERT INTO ?t ('.$this->MakeSQLFromArray($data, ',', 'k')
				.') VALUES('.$this->MakeSQLFromArray($data, ',', 'v').')'
				.' ON DUPLICATE KEY UPDATE '.$this->MakeSQLFromArray($data, ', ');
		if ($this->transaction)
		{
			try
			{
				$result = $this->query($sql, array($table));
				$this->rollback = !$result;
			} catch (DBException $e) {
				$this->rollback = true;
			}
			if ($this->rollback) {
				throw new DBExceptionRollbackTrigger();
			}
		}
		return $result;
	}
}

class DBExceptionTransaction extends DBException
{
	public function __construct($message, $errno, $error)
	{
        parent::__construct($message, $errno);
        $this->error = $error;
	}
	
    public function getError() {return $this->error;}
    
    public function __toString()
    {
    	return 'DB Error. '.htmlSpecialChars($this->getMessage())
    			.'. ('.$this->errno.') "'.htmlSpecialChars($this->getError()).'"';
    }
	
    protected $error;
}

class DBExceptionTransactionFailed extends DBExceptionTransaction
{
	public function __construct($errno, $error)
	{
		parent::__construct('Transaction start failed', $errno, $error);
	}
}

class DBExceptionRollbackFailed extends DBExceptionTransaction
{
	public function __construct($errno, $error)
	{
		parent::__construct('Transaction rollback failed', $errno, $error);
	}
}

class DBExceptionCommitFailed extends DBExceptionTransaction
{
	public function __construct($errno, $error)
	{
		parent::__construct('Transaction commit failed', $errno, $error);
	}
}

class DBExceptionRollbackTrigger extends DBException {}

?>